import inspect
from typing import Dict, Annotated, get_origin, get_args

TYPE_MAPPING = {
    "str": "string",
    "int": "integer",
    "float": "number",
    "bool": "boolean",
    "list": "array",
    "dict": "object",
    # 添加其他类型的映射，如果需要的话
}


def gen_func_description(func) -> Dict:
    # 生成函数描述
    func_name = func.__name__
    func_desc = func.__doc__.strip() if func.__doc__ else ""

    # 获取函数参数的注解信息和默认值
    sig = inspect.signature(func)
    annotations = func.__annotations__

    parameters = {}
    required_params = []

    for name, param in sig.parameters.items():
        hint = annotations.get(name, None)
        origin = get_origin(hint)
        args = get_args(hint)

        if origin is Annotated:
            # 处理 Annotated 类型
            param_type = args[0].__name__.lower()
            param_desc = args[1]
        else:
            # 处理非 Annotated 类型
            param_type = (
                hint.__name__.lower() if hasattr(hint, "__name__") else str(hint)
            )
            param_desc = ""

        # 使用类型映射进行转换
        param_type = TYPE_MAPPING.get(param_type, "string")

        parameters[name] = {"description": param_desc, "type": param_type}

        # 检查参数默认值
        if param.default is param.empty:
            required_params.append(name)

    # 将生成的参数信息结构化
    param_schema = {
        "type": "object",
        "properties": parameters,
        "required": required_params,
    }

    # 返回最终的结构化函数描述
    return {
        "type": "function",
        "function": {
            "name": func_name,
            "description": func_desc,
            "parameters": param_schema,
        },
    }


def repair_spec(
    suggestion: Annotated[str, "spec脚本修改建议"],
    fault_segment: Annotated[str, "spec脚本中导致错误的代码片段,用于定位错误位置"] = "",
    repaired_segment: Annotated[
        str, "修复后的spec脚本代码段,用于替换原有的错误片段"
    ] = "",
) -> str:
    """根据报错信息修复spec脚本"""

    return "spec 脚本修复"


def repair_spec_impl(
    original_spec_file: str,
    fault_segment: str,
    repaired_segment: str,
    repaired_spec_file: str,
) -> bool:
    with open(original_spec_file, "r", encoding="utf-8", errors="ignore") as f:
        spec_lines = f.readlines()
    fault_lines = fault_segment.split("\n")
    fault_lines = [line + "\n" for line in fault_lines]
    repaired_lines = repaired_segment.split("\n")
    repaired_lines = [line + "\n" for line in repaired_lines]

    indices = [index for index, line in enumerate(spec_lines) if line == fault_lines[0]]

    if len(indices) == 0:
        return False

    success_list = [True] * len(indices)
    fault_length = len(fault_lines)
    for i in range(len(indices)):
        index = indices[i]
        fault_segment_extract = spec_lines[index : index + fault_length]
        for j in range(fault_length):
            if fault_segment_extract[j] != fault_lines[j]:
                success_list[i] = False
                break

    if any(success_list):
        # 找到第一个匹配的索引
        index = indices[success_list.index(True)]
        del spec_lines[index : index + fault_length]
        spec_lines[index:index] = repaired_segment
    else:
        return False

    with open(repaired_spec_file, "w") as f:
        f.write("".join(spec_lines))
    return True



import os
import re
import json
from openai import OpenAI
from .utils import gen_func_description, repair_spec, repair_spec_impl

SYSTEM_PROMPT = "你是一位经验丰富的openEuler RPM软件包构建人员，你的任务是根据提供的spec脚本和报错日志修复spec脚本，以解决构建过程中出现的问题。"

PROMPT_TEMPLATE = """
spec脚本：
{spec}

报错日志：
{log}
"""


class SpecBot:
    def __init__(self):
        api_key = os.getenv("OPENAI_API_KEY", None)
        base_url = os.getenv("OPENAI_BASE_URL", None)
        self.client = OpenAI(api_key=api_key, base_url=base_url)
        self.model = "gpt-4-0613"

    def repair(self, spec_file, log_file, repair_file):
        spec = self._preprocess_spec(spec_file)
        log = self._preprocess_log(log_file)
        tools = self._prepare_tools()
        messages = self._prepare_messages(spec, log)
        
        is_repaired = False
        try:
            response = self.client.chat.completions.create(
                model=self.model,
                messages=messages,
                tools=tools,
                tool_choice={"type": "function", "function": {"name": "repair_spec"}},
            )
            tool_calls = response.choices[0].message.tool_calls
            arguments = tool_calls[0].function.arguments
            arguments = json.loads(arguments)
            suggestion = arguments.get("suggestion", None)
            fault_segment = arguments.get("fault_segment", None)
            repaired_segment = arguments.get("repaired_segment", None)
            
            if suggestion and fault_segment and repaired_segment:
                is_repaired = repair_spec_impl(
                    spec_file, fault_segment, repaired_segment, repair_file
                )
        except Exception as e:
            suggestion = str(e)
        return suggestion, is_repaired

    def _prepare_messages(self, spec, log):
        # 准备消息
        messages = []
        if SYSTEM_PROMPT:
            messages.append({"role": "system", "content": SYSTEM_PROMPT})
        messages.append(
            {"role": "user", "content": PROMPT_TEMPLATE.format(spec=spec, log=log)}
        )
        return messages

    def _prepare_tools(self):
        # 准备工具
        return [gen_func_description(repair_spec)]

    def _preprocess_spec(self, spec_file):
        # 预处理spec
        with open(spec_file, "r", encoding="utf-8", errors="ignore") as f:
            lines = f.readlines()
        start_index = 0
        for i in range(len(lines)):
            if "License" in lines[i]:
                start_index = i + 1
                break
            if "BuildRequires" in lines[i]:
                start_index = i
                break
        spec = "".join(lines[start_index:])
        return spec

    def _preprocess_log(self, log_file):
        # 预处理log
        with open(log_file, "r", encoding="utf-8", errors="ignore") as f:
            lines = f.readlines()
        start_index = 0
        end_index = len(lines)

        for i in range(len(lines) - 1, -1, -1):
            if "RPM build errors:" in lines[i]:
                end_index = i

            pattern = re.compile(r"^Executing\(%\w+\):")
            if pattern.match(lines[i]):
                start_index = i
                break

        log = "".join(lines[start_index:end_index])

        return log



from setuptools import setup, find_packages

setup(
    name="repair-spec",
    version="0.1.0.dev3",
    package_dir={"": "src"},
    packages=find_packages("src"),
    install_requires=[
        "openai",
    ],
    author="Sibo Zhang",
    author_email="zhangsibo1129@gmail.com",
    description="Auto repair spec",
    url="https://github.com/zhangsibo1129/specrepair",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
)
